"""
    effectus.helpers
    ~~~~~~~~~~~~~~~~

    The building blocks for the core functions.
 
    :copyright: (c) 2017 by Benjamin Weber
    :license: MIT, see LICENSE for more details.
"""

from math import gcd
from collections import Counter

from quicktions import Fraction

from effectus.pareto import ratio

def decimal_gcd(num1, num2):
    """Return greatest common denominator for decimal numbers
    with leading zero.

    Args:
        num1 (Fraction): Number.
        num2 (Fraction): Number.

    Returns:
        Fraction: Greatest common denominator.

    Example:
        >>> decimal_gcd(Fraction(1, 50), Fraction(1, 25))
        Fraction(1, 50)
    """
    gcd_value = gcd(int(num1*100), int(num2*100))
    if gcd_value:
        return Fraction(gcd_value, 100).limit_denominator(100)

def sort_effects(effect_values):
    """Sort values descending.

    Args:
        effect_values (List[number]): Effects as list of numbers.

    Returns:
        List[number]: Sorted list of effect values with decreasing value.

    Example:
        >>> sort_effects([65, 27, 15, 621, 109, 30, 45, 789, 9, 12])
        [789, 621, 109, 65, 45, 30, 27, 15, 12, 9]
    """
    sorted_effects = list(effect_values)
    sorted_effects.sort()
    sorted_effects.reverse()
    return sorted_effects

def find_nearest(num, number_list):
    """Which number in number_list is nearest to number?

    Args:
        num (number): Value to find nearest value for in ``number_list``.
        number_list (List[Fraction]): Predefined list of 0 < Fractions < 1.

    Returns:
        float: Nearest value in `number_list` for `num`.

    Example:
        >>> find_nearest(0.2, [Fraction(1, 5), Fraction(1, 6)])
        Fraction(1, 6)
    """
    prev = None
    prev_delta = 0
    for ordinal, compare_to in enumerate(number_list):
        delta = abs(num-compare_to)
        if not prev or delta < prev_delta:
            prev_delta = delta
            prev = ordinal
    return number_list[prev]

def decimal_range(lower, upper, decimal_places):
    """Return list of decimals in the range of lower and upper.
    The step between two neighbouring is 1/10**decimal_places.
    """
    result = []
    # move numbers to the left of the decimal point
    working_step = int(Fraction(10**decimal_places, 10))
    # 10**2 == 100
    #Fraction(100, 1)
    working_lower = int(Fraction.from_float(lower) * 10**decimal_places)
    working_upper = int(Fraction.from_float(upper) * 10**decimal_places)
    # use working_upper + working_step as real_upper bound as range
    # otherwise would not include working_upper
    for ordinal in range(working_lower, working_upper+working_step, working_step):
        # move numbers back to the right of the decimal point
        result.append(Fraction(ordinal, 10**decimal_places))
    return result

def make_relations(effect_values):
    """Create lost of tuples (cumulative_causes, cumulative_effects)
    from effects.

    Args:
        effect_values (List[number]): Effects as list of numbers.

    Returns:
        A list of tuples with cause-effect mappings, for
        example::

        [(Fraction(1, 10), Fraction(263, 574)), ...]

    Example:
        >>> make_relations([789, 621, 109, 65, 45, 30, 27, 15, 12, 9])
        [(Fraction(1, 10), Fraction(263, 574)),
         (Fraction(1, 5), Fraction(235, 287)),
         (Fraction(3, 10), Fraction(217, 246)),
         (Fraction(2, 5), Fraction(264, 287)),
         (Fraction(1, 2), Fraction(543, 574)),
         (Fraction(3, 5), Fraction(79, 82)),
         (Fraction(7, 10), Fraction(281, 287)),
         (Fraction(4, 5), Fraction(81, 82)),
         (Fraction(9, 10), Fraction(571, 574)),
         (Fraction(1, 1), Fraction(1, 1))]
    """
    relations = []
    count = len(effect_values)
    total = sum(effect_values)
    runner = 0
    for ordinal, value in enumerate(sort_effects(effect_values), 1):
        runner += value
        relations.append((Fraction(ordinal, count),
                          Fraction(Fraction.from_float(runner),
                                   Fraction.from_float(total))))
    return relations

def greatest_gcd(effect_values):
    """Determine cause-effect relationship with greatest common denominator.
    But only for causes_ratio <= 0.5 and effects_ratio >= 0.5.

    Args:
        effect_values (List[number]): Effects as list of numbers.

    Returns:
        A list of tuples of cause-effect mappings, for
        example::

        [(Fraction(3, 10), Fraction(217, 246))]

    Example:
        >>> greatest_gcd([789, 621, 109, 65, 45, 30, 27, 15, 12, 9])
        (Fraction(3, 10), Fraction(217, 246))
    """
    effects_count = len(effect_values)
    sorted_effects = sort_effects(effect_values)
    total = sum(effect_values)
    cml_sum = 0
    peak_gcd = None
    peak_causes = None
    peak_effects = None
    for ordinal, effect_value in enumerate(sorted_effects):
        causes_ratio = Fraction(ordinal + 1, effects_count)
        cml_sum += effect_value
        effects_ratio = Fraction(Fraction.from_float(cml_sum),
                                 Fraction.from_float(total))
        if causes_ratio > 0.5 or effects_ratio < 0.5:
            continue
        the_gcd = decimal_gcd(causes_ratio, effects_ratio)
        if peak_gcd is None or the_gcd > peak_gcd:
            peak_gcd = the_gcd
            peak_causes = causes_ratio
            peak_effects = effects_ratio
    if peak_causes and peak_effects:
        return peak_causes, peak_effects

def get_relevant(relations, causes_list=None, effects_list=None):
    """Return only relevant variations of causes and effects as tuples.

    Args:
        relations: A list of tuples with cause-effect mappings. This is the
          output of ``make_relations``.
        causes_list: A list of causes that are relevant. A cause may be any
            value between 0 and 1.
        effects_list: A list of effects that are relevant. An effect may be any
            value between 0 and 1.

    Returns:
        A list of tuples with cause-effect mappings with total_delta and
        greatest common decimal denominator, for example::

        [(Fraction(1, 5), Fraction(4, 5), \
Fraction(27, 1175), Fraction(1, 5))]

    Example:
        >>> relations = make_relations([789, 621, 109, 65, 45, 30, 27, 15, 12,
        9])
        >>> get_relevant(relations)
        [(Fraction(1, 5), Fraction(4, 5),
        Fraction(27, 1175), Fraction(1, 5)), (Fraction(3, 10), Fraction(9, 10),
        Fraction(22, 1085), Fraction(3, 10)), (Fraction(2, 5),
        Fraction(9, 10), Fraction(19, 880), Fraction(1, 10))]
    """
    causes_list = causes_list or [Fraction(1, 100), Fraction(5, 100),
                                  Fraction(1, 10), Fraction(3, 10),
                                  Fraction(1, 5), Fraction(2, 5),
                                  Fraction(3, 5),
                                  Fraction(1, 4),
                                  Fraction(1, 3),
                                  Fraction(1, 2)
                                 ]
    effects_list = effects_list or [Fraction(95, 100),
                                    Fraction(9, 10),
                                    Fraction(3, 5), Fraction(4, 5),
                                    Fraction(2, 3),
                                    Fraction(1, 2)
                                   ]
    result = list()
    for given_causes, given_effects in relations:
        # skip if causes > 0.5 or effects < than 0.5
        if given_causes + Fraction(2, 100) > Fraction(1, 2) or \
           given_effects - Fraction(2, 100) < Fraction(1, 2):
            continue
        else:
            nearest_causes = find_nearest(given_causes, causes_list)
            nearest_effects = find_nearest(given_effects, effects_list)
            delta_causes = abs(nearest_causes - given_causes) / given_causes
            delta_effects = abs(nearest_effects - given_effects) / given_effects
            total_delta = delta_causes + delta_effects
            if total_delta <= 0.05:
                result.append((nearest_causes,
                               nearest_effects,
                               total_delta,
                               decimal_gcd(nearest_causes, nearest_effects))
                             )
    return result

def pick_best(relevant_relations):
    """Pick the single relation with the highest power.

    Args:
        relevant_relations: A list of relevant cause-effect
          relations that is tuple (causes, effects, total_delta,
          gcd). This is the output from ``get_relevant``.

    Returns:
        A dict with the most relevant cause-effect relationship.

    Example:
        >>> relations = make_relations([789, 621, 109, 65, 45, 30, \
27, 15, 12, 9])
        >>> relevant = get_relevant(relations)
        >>> pick_best(relevant)
        {'causes': Fraction(1, 5),
         'effects': Fraction(4, 5),
         'gcd': Fraction(1, 5),
         'variability': 0.02297872340425532}
    """
    prev_power = None
    pick = {}
    for causes, effects, delta, the_gcd in relevant_relations:
        if causes + effects == 1:
            power = effects / causes * 2
        else:
            power = effects / causes
        if prev_power is None or power > prev_power:
            pick = {'causes': causes,
                    'effects': effects,
                    'gcd': the_gcd,
                    'variability': float(delta)
                   }
        prev_power = power
    if pick != {}:
        return pick

def format_summary(summary, effect_values, fracs):
    """Extends summary dict by ``pareto`` and ``ratio``, fracifies or rounds it.

    Args:
        summary (dict): A summary created by ``pick_best`` or ``greatest_gcd``.
        effect_values (List[number]): Effects as list of numbers.
        fracs (bool): True to return fraction format, otherwise float.

    Returns:
        If a cause-effect relationship other than
        1:1 is present, a dict with keys `pareto`, `ratio`,
        `causes`, `effects` and optionally `variability`.

    Example:
        >>> format_summary({'causes': Fraction(1, 5),
                            'effects': Fraction(4, 5),
                            'gcd': Fraction(1, 5),
                            'variability': 0.02297872340425532})
        {'causes': '1/5',
         'effects': '4/5',
         'pareto': True,
         'ratio': 0.707,
         'variability': 0.02}
    """
    if 'gcd' in summary.keys():
        summary.pop('gcd')

    summary['ratio'] = round(ratio(effect_values), 3)
    summary['pareto'] = summary['ratio'] <= 1

    if 'variability' in summary.keys():
        summary['variability'] = round(summary['variability'], 2)
    if fracs:
        summary['causes'] = '{}/{}'.format(summary['causes'].numerator,
                                           summary['causes'].denominator)
    else:
        summary['causes'] = round(float(summary['causes']), 3)
    if fracs:
        summary['effects'] = '{}/{}'.format(summary['effects'].numerator,
                                            summary['effects'].denominator)
    else:
        summary['effects'] = round(float(summary['effects']), 3)
    return summary

# Value Handling

def reduce_values(values):
    """Reduces a list of values into a value-frequency mapping.

    Args:
        values (List[number]): Effects as list of numbers.

    Returns:
        A list of tuples of like [(value, frequency), ...].

    Example:
        >>> reduce_values([1,1,1,3,2])
        [(1, 3), (3, 2)]
    """
    vafreqs = Counter()

    for value in values:
        vafreqs[value] += 1

    outlist = []
    for value, freq in vafreqs.items():
        outlist.append('{},{}'.format(value, freq))
    return outlist

def write_reduced_values(values, filename):
    """Write value-frequency mappings to file.

    Args:
        values (List[number]): Effects as list of numbers.
        filename: File path to write to.

    Return:
        Nothing. The value-frequency mappings are written
        separated by comma line-wise to the given filename.

    Example:
        >>> write_reduced_values([1,1,1,1,3,3])
    """
    outlist = reduce_values(values)
    with open(filename, 'w') as target:
        target.write('\n'.join(outlist))

def get_values(csv_reader, observations_col=1, frequency_col=2):
    """Get values from csv file.

    Args:
        csv_reader:
        observations_col (int): Column number of observations (default: 1).
        frequency_col (int): Column number of frequency (default: 2).
          if < 0 or csv file has only one column, values will only
          be considered.

    Returns:
        A list of observation-frequency tuples
        like::

        [(23.3, 1), (80.1, 3), (99.3, 2)]
    """
    result = []
    for row in csv_reader:
        try:
            observation = float(row[observations_col-1])
            frequency = 1
        except (ValueError, IndexError):
            continue
        if len(row) > 1:
            try:
                if frequency_col < 0:
                    frequency = 1
                else:
                    frequency = int(row[frequency_col-1])
            except (ValueError, IndexError):
                raise ValueError
        else:
            frequency = 1
        result.append((observation, frequency))
    return result

def unfold_values(values):
    """Unfold list of observation-frequency mappings.

    Args:
        values (Tuple[number, int]): observation-frequency mapping.

    Returns:
        List[number]: List of values.

    Example:
        >>> unfold_values([(1, 3), (2, 1), (3, 1)])
        [1, 1, 1, 2, 3]
    """
    result = []
    for observation, frequency in values:
        result += [observation] * frequency
    return result
